"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.renderAccount = void 0;
const render_enums_1 = require("./render-enums");
const serdes_1 = require("./serdes");
const type_mapper_1 = require("./type-mapper");
const types_1 = require("./types");
const utils_1 = require("./utils");
function colonSeparatedTypedField(field, prefix = '') {
    return `${prefix}${field.name}: ${field.tsType}`;
}
class AccountRenderer {
    constructor(account, hasImplicitDiscriminator, typeMapper) {
        this.account = account;
        this.hasImplicitDiscriminator = hasImplicitDiscriminator;
        this.typeMapper = typeMapper;
        this.upperCamelAccountName = account.name
            .charAt(0)
            .toUpperCase()
            .concat(account.name.slice(1));
        this.camelAccountName = account.name
            .charAt(0)
            .toLowerCase()
            .concat(account.name.slice(1));
        this.accountDataClassName = `${this.upperCamelAccountName}AccountData`;
        this.accountDataArgsTypeName = `${this.accountDataClassName}Args`;
        this.dataStructName = `${this.camelAccountName}AccountDataStruct`;
        this.accountDiscriminatorName = `${this.camelAccountName}AccountDiscriminator`;
    }
    serdeProcess() {
        return this.typeMapper.mapSerdeFields(this.account.type.fields);
    }
    // -----------------
    // Rendered Fields
    // -----------------
    getTypedFields() {
        return this.account.type.fields.map((f) => {
            const tsType = this.typeMapper.map(f.type, f.name);
            return { name: f.name, tsType };
        });
    }
    getPrettyFields() {
        return this.account.type.fields.map((f) => {
            const postfix = f.type === 'publicKey' ? '.toBase58()' : '';
            return `${f.name}: this.${f.name}${postfix}`;
        });
    }
    // -----------------
    // Imports
    // -----------------
    renderImports() {
        const imports = this.typeMapper.importsForSerdePackagesUsed(new Set([types_1.SOLANA_WEB3_PACKAGE, types_1.BEET_PACKAGE]));
        return imports.join('\n');
    }
    // -----------------
    // Account Args
    // -----------------
    renderAccountDataArgsType(fields) {
        const renderedFields = fields
            .map((f) => colonSeparatedTypedField(f))
            .join('\n  ');
        return `/**
 * Arguments used to create {@link ${this.accountDataClassName}}
 */
export type ${this.accountDataArgsTypeName} = {
  ${renderedFields}
}`;
    }
    renderByteSizeMethods() {
        if (this.typeMapper.usedFixableSerde) {
            const byteSizeValue = this.hasImplicitDiscriminator
                ? `{
      accountDiscriminator: ${this.accountDiscriminatorName},
      ...instance,
    }`
                : `instance`;
            return `
  /**
   * Returns the byteSize of a {@link Buffer} holding the serialized data of
   * {@link ${this.accountDataClassName}} for the provided args.
   *
   * @param args need to be provided since the byte size for this account
   * depends on them
   */
  static byteSize(args: ${this.accountDataArgsTypeName}) {
    const instance = ${this.accountDataClassName}.fromArgs(args)
    return ${this.dataStructName}.toFixedFromValue(${byteSizeValue}).byteSize
  }

  /**
   * Fetches the minimum balance needed to exempt an account holding 
   * {@link ${this.accountDataClassName}} data from rent
   *
   * @param args need to be provided since the byte size for this account
   * depends on them
   * @param connection used to retrieve the rent exemption information
   */
  static async getMinimumBalanceForRentExemption(
    args: ${this.accountDataArgsTypeName},
    connection: web3.Connection,
    commitment?: web3.Commitment
  ): Promise<number> {
    return connection.getMinimumBalanceForRentExemption(
      ${this.accountDataClassName}.byteSize(args),
      commitment
    )
  }
  `.trim();
        }
        else {
            return `
  /**
   * Returns the byteSize of a {@link Buffer} holding the serialized data of
   * {@link ${this.accountDataClassName}}
   */
  static get byteSize() {
    return ${this.dataStructName}.byteSize;
  }

  /**
   * Fetches the minimum balance needed to exempt an account holding 
   * {@link ${this.accountDataClassName}} data from rent
   *
   * @param connection used to retrieve the rent exemption information
   */
  static async getMinimumBalanceForRentExemption(
    connection: web3.Connection,
    commitment?: web3.Commitment,
  ): Promise<number> {
    return connection.getMinimumBalanceForRentExemption(
      ${this.accountDataClassName}.byteSize,
      commitment,
    );
  }

  /**
   * Determines if the provided {@link Buffer} has the correct byte size to
   * hold {@link ${this.accountDataClassName}} data.
   */
  static hasCorrectByteSize(buf: Buffer, offset = 0) {
    return buf.byteLength - offset === ${this.accountDataClassName}.byteSize;
  }
      `.trim();
        }
    }
    // -----------------
    // AccountData class
    // -----------------
    renderAccountDiscriminatorVar() {
        if (!this.hasImplicitDiscriminator)
            return '';
        const accountDisc = JSON.stringify(Array.from((0, utils_1.accountDiscriminator)(this.account.name)));
        return `const ${this.accountDiscriminatorName} = ${accountDisc}`;
    }
    renderAccountDataClass(fields) {
        const constructorArgs = fields
            .map((f) => colonSeparatedTypedField(f, 'readonly '))
            .join(',\n    ');
        const constructorParams = fields
            .map((f) => `args.${f.name}`)
            .join(',\n      ');
        const prettyFields = this.getPrettyFields().join(',\n      ');
        const byteSizeMethods = this.renderByteSizeMethods();
        const accountDiscriminatorVar = this.renderAccountDiscriminatorVar();
        const serializeValue = this.hasImplicitDiscriminator
            ? `{ 
      accountDiscriminator: ${this.accountDiscriminatorName},
      ...this
    }`
            : 'this';
        return `
${accountDiscriminatorVar};
/**
 * Holds the data for the {@link ${this.upperCamelAccountName}Account} and provides de/serialization
 * functionality for that data
 */
export class ${this.accountDataClassName} implements ${this.accountDataArgsTypeName} {
  private constructor(
    ${constructorArgs}
  ) {}

  /**
   * Creates a {@link ${this.accountDataClassName}} instance from the provided args.
   */
  static fromArgs(args: ${this.accountDataArgsTypeName}) {
    return new ${this.accountDataClassName}(
      ${constructorParams}
    );
  }

  /**
   * Deserializes the {@link ${this.accountDataClassName}} from the data of the provided {@link web3.AccountInfo}.
   * @returns a tuple of the account data and the offset up to which the buffer was read to obtain it.
   */
  static fromAccountInfo(
    accountInfo: web3.AccountInfo<Buffer>,
    offset = 0
  ): [ ${this.accountDataClassName}, number ]  {
    return ${this.accountDataClassName}.deserialize(accountInfo.data, offset)
  }

  /**
   * Deserializes the {@link ${this.accountDataClassName}} from the provided data Buffer.
   * @returns a tuple of the account data and the offset up to which the buffer was read to obtain it.
   */
  static deserialize(
    buf: Buffer,
    offset = 0
  ): [ ${this.accountDataClassName}, number ]{
    return ${this.dataStructName}.deserialize(buf, offset);
  }

  /**
   * Serializes the {@link ${this.accountDataClassName}} into a Buffer.
   * @returns a tuple of the created Buffer and the offset up to which the buffer was written to store it.
   */
  serialize(): [ Buffer, number ] {
    return ${this.dataStructName}.serialize(${serializeValue})
  }

  ${byteSizeMethods}

  /**
   * Returns a readable version of {@link ${this.accountDataClassName}} properties
   * and can be used to convert to JSON and/or logging
   */
  pretty() {
    return {
      ${prettyFields}
    };
  }
}`.trim();
    }
    // -----------------
    // Struct
    // -----------------
    renderDataStruct(fields) {
        let discriminatorName;
        let discriminatorField;
        let discriminatorType;
        if (this.hasImplicitDiscriminator) {
            discriminatorName = 'accountDiscriminator';
            discriminatorField = this.typeMapper.mapSerdeField((0, utils_1.anchorDiscriminatorField)('accountDiscriminator'));
            discriminatorType = (0, utils_1.anchorDiscriminatorType)(this.typeMapper, `account ${this.account.name} discriminant type`);
        }
        return (0, serdes_1.renderDataStruct)({
            fields,
            structVarName: this.dataStructName,
            className: this.accountDataClassName,
            argsTypename: this.accountDataArgsTypeName,
            discriminatorName,
            discriminatorField,
            discriminatorType,
            isFixable: this.typeMapper.usedFixableSerde,
        });
    }
    render() {
        this.typeMapper.clearUsages();
        const typedFields = this.getTypedFields();
        const beetFields = this.serdeProcess();
        const enums = (0, render_enums_1.renderScalarEnums)(this.typeMapper.scalarEnumsUsed).join('\n');
        const imports = this.renderImports();
        const accountDataArgsType = this.renderAccountDataArgsType(typedFields);
        const accountDataClass = this.renderAccountDataClass(typedFields);
        const dataStruct = this.renderDataStruct(beetFields);
        return `${imports}

${enums}

${accountDataArgsType}

${accountDataClass}

${dataStruct}`;
    }
}
function renderAccount(account, forceFixable, userDefinedEnums, hasImplicitDiscriminator) {
    const typeMapper = new type_mapper_1.TypeMapper(forceFixable, userDefinedEnums);
    const renderer = new AccountRenderer(account, hasImplicitDiscriminator, typeMapper);
    return renderer.render();
}
exports.renderAccount = renderAccount;
//# sourceMappingURL=render-account.js.map