"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.renderType = exports.enumVarNameFromTypeName = exports.structVarNameFromTypeName = void 0;
const type_mapper_1 = require("./type-mapper");
const types_1 = require("./types");
const assert_1 = require("assert");
const serdes_1 = require("./serdes");
const render_enums_1 = require("./render-enums");
function structVarNameFromTypeName(ty) {
    const camelTyName = ty.charAt(0).toLowerCase().concat(ty.slice(1));
    return `${camelTyName}Struct`;
}
exports.structVarNameFromTypeName = structVarNameFromTypeName;
function enumVarNameFromTypeName(ty) {
    const camelTyName = ty.charAt(0).toLowerCase().concat(ty.slice(1));
    return `${camelTyName}Enum`;
}
exports.enumVarNameFromTypeName = enumVarNameFromTypeName;
class TypeRenderer {
    constructor(ty, typeMapper = new type_mapper_1.TypeMapper()) {
        this.ty = ty;
        this.typeMapper = typeMapper;
        this.userDefinedEnums = new Set();
        // -----------------
        // Rendered Fields
        // -----------------
        this.renderTypeField = (field) => {
            const typescriptType = this.typeMapper.map(field.type, field.name);
            return `${field.name}: ${typescriptType}`;
        };
        this.upperCamelTyName = ty.name
            .charAt(0)
            .toUpperCase()
            .concat(ty.name.slice(1));
        this.camelTyName = ty.name.charAt(0).toLowerCase().concat(ty.name.slice(1));
        this.structArgName = structVarNameFromTypeName(ty.name);
        this.enumArgName = enumVarNameFromTypeName(ty.name);
    }
    renderTypeScriptType() {
        if ((0, types_1.isIdlTypeEnum)(this.ty.type)) {
            this.userDefinedEnums.add(this.ty.name);
            return (0, render_enums_1.renderScalarEnum)(this.ty.name, this.ty.type.variants.map((x) => x.name), true);
        }
        if (this.ty.type.fields.length === 0)
            return '';
        const fields = this.ty.type.fields
            .map((field) => this.renderTypeField(field))
            .join(',\n  ');
        const code = `export type ${this.upperCamelTyName} = {
  ${fields}
}`;
        return code;
    }
    // -----------------
    // Imports
    // -----------------
    renderImports() {
        const imports = this.typeMapper.importsForSerdePackagesUsed();
        return imports.join('\n');
    }
    // -----------------
    // Data Struct
    // -----------------
    renderDataStructOrEnum() {
        if ((0, types_1.isIdlTypeEnum)(this.ty.type)) {
            this.userDefinedEnums.add(this.ty.name);
            const serde = this.typeMapper.mapSerde(this.ty.type, this.ty.name);
            const enumTy = this.typeMapper.map(this.ty.type, this.ty.name);
            this.typeMapper.serdePackagesUsed.add(types_1.BEET_PACKAGE);
            const exp = (0, serdes_1.serdePackageExportName)(types_1.BEET_PACKAGE);
            // Need the cast here since otherwise type is assumed to be
            // FixedSizeBeet<typeof ${enumTy}, typeof ${enumTy}> which is incorrect
            return `const ${this.enumArgName} = ${serde} as ${exp}.FixedSizeBeet<${enumTy}, ${enumTy}>`;
        }
        const mappedFields = this.typeMapper.mapSerdeFields(this.ty.type.fields);
        return (0, serdes_1.renderTypeDataStruct)({
            fields: mappedFields,
            structVarName: this.structArgName,
            typeName: this.upperCamelTyName,
            isFixable: this.typeMapper.usedFixableSerde,
        });
    }
    render() {
        this.typeMapper.clearUsages();
        const kind = this.ty.type.kind;
        (0, assert_1.strict)(kind === 'struct' || kind === 'enum', `only user defined structs or enums are supported, ${this.ty.name} is of type ${this.ty.type.kind}`);
        const typeScriptType = this.renderTypeScriptType();
        const dataStruct = this.renderDataStructOrEnum();
        const imports = this.renderImports();
        return `
${imports}
${typeScriptType}
export ${dataStruct}
`.trim();
    }
}
function renderType(ty) {
    const renderer = new TypeRenderer(ty);
    const code = renderer.render();
    const isFixable = renderer.typeMapper.usedFixableSerde;
    const userDefinedEnums = renderer.userDefinedEnums;
    return { code, isFixable, userDefinedEnums };
}
exports.renderType = renderType;
//# sourceMappingURL=render-type.js.map