"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Solita = void 0;
const fs_1 = require("fs");
const path_1 = __importDefault(require("path"));
// import { renderAccount } from './render-account'
const render_errors_1 = require("./render-errors");
const render_instruction_1 = require("./render-instruction");
const types_1 = require("./types");
const utils_1 = require("./utils");
const prettier_1 = require("prettier");
const render_type_1 = require("./render-type");
const render_account_1 = require("./render-account");
__exportStar(require("./types"), exports);
function renderIndex(modules) {
    return modules.map((x) => `export * from './${x}';`).join('\n');
}
const DEFAULT_FORMAT_OPTS = {
    semi: false,
    singleQuote: true,
    trailingComma: 'es5',
    useTabs: false,
    tabWidth: 2,
    arrowParens: 'always',
    printWidth: 80,
    parser: 'typescript',
};
class Solita {
    constructor(idl, { formatCode = false, formatOpts = {}, } = {}) {
        this.idl = idl;
        this.formatCode = formatCode;
        this.formatOpts = { ...DEFAULT_FORMAT_OPTS, ...formatOpts };
        this.accountsHaveImplicitDiscriminator = !(0, types_1.isShankIdl)(idl);
    }
    renderCode() {
        var _a, _b, _c, _d, _e, _f;
        const programId = this.idl.metadata.address;
        const fixableTypes = new Set();
        let userDefinedEnums = new Set();
        function forceFixable(ty) {
            if ((0, types_1.isIdlTypeDefined)(ty) && fixableTypes.has(ty.defined)) {
                return true;
            }
            return false;
        }
        // NOTE: we render types first in order to know which ones are 'fixable' by
        // the time we render accounts and instructions
        const types = {};
        (0, utils_1.logDebug)('Rendering %d types', (_b = (_a = this.idl.types) === null || _a === void 0 ? void 0 : _a.length) !== null && _b !== void 0 ? _b : 0);
        if (this.idl.types != null) {
            for (const ty of this.idl.types) {
                (0, utils_1.logDebug)(`Rendering type ${ty.name}`);
                (0, utils_1.logTrace)('kind: %s', ty.type.kind);
                if ((0, types_1.isIdlDefinedType)(ty.type)) {
                    (0, utils_1.logTrace)('fields: %O', ty.type.fields);
                }
                else {
                    if ((0, types_1.isIdlTypeEnum)(ty.type)) {
                        (0, utils_1.logTrace)('variants: %O', ty.type.variants);
                    }
                }
                let { code, isFixable, userDefinedEnums: enums } = (0, render_type_1.renderType)(ty);
                userDefinedEnums = enums;
                if (isFixable) {
                    fixableTypes.add(ty.name);
                }
                if (this.formatCode) {
                    try {
                        code = (0, prettier_1.format)(code, this.formatOpts);
                    }
                    catch (err) {
                        console.error(`Failed to format ${ty.name} instruction`);
                        console.error(err);
                    }
                }
                types[ty.name] = code;
            }
        }
        const instructions = {};
        for (const ix of this.idl.instructions) {
            (0, utils_1.logDebug)(`Rendering instruction ${ix.name}`);
            (0, utils_1.logTrace)('args: %O', ix.args);
            (0, utils_1.logTrace)('accounts: %O', ix.accounts);
            let code = (0, render_instruction_1.renderInstruction)(ix, programId, forceFixable, userDefinedEnums);
            if (this.formatCode) {
                try {
                    code = (0, prettier_1.format)(code, this.formatOpts);
                }
                catch (err) {
                    console.error(`Failed to format ${ix.name} instruction`);
                    console.error(err);
                }
            }
            instructions[ix.name] = code;
        }
        const accounts = {};
        for (const account of (_c = this.idl.accounts) !== null && _c !== void 0 ? _c : []) {
            (0, utils_1.logDebug)(`Rendering account ${account.name}`);
            (0, utils_1.logTrace)('type: %O', account.type);
            let code = (0, render_account_1.renderAccount)(account, forceFixable, userDefinedEnums, this.accountsHaveImplicitDiscriminator);
            if (this.formatCode) {
                try {
                    code = (0, prettier_1.format)(code, this.formatOpts);
                }
                catch (err) {
                    console.error(`Failed to format ${account.name} account`);
                    console.error(err);
                }
            }
            accounts[account.name] = code;
        }
        (0, utils_1.logDebug)('Rendering %d errors', (_e = (_d = this.idl.errors) === null || _d === void 0 ? void 0 : _d.length) !== null && _e !== void 0 ? _e : 0);
        let errors = (0, render_errors_1.renderErrors)((_f = this.idl.errors) !== null && _f !== void 0 ? _f : []);
        if (errors != null && this.formatCode) {
            try {
                errors = (0, prettier_1.format)(errors, this.formatOpts);
            }
            catch (err) {
                console.error(`Failed to format errors`);
                console.error(err);
            }
        }
        return { instructions, accounts, types, errors };
    }
    async renderAndWriteTo(outputDir) {
        const { instructions, accounts, types, errors } = this.renderCode();
        await this.writeInstructions(outputDir, instructions);
        if (Object.keys(accounts).length !== 0) {
            await this.writeAccounts(outputDir, accounts);
        }
        if (Object.keys(types).length !== 0) {
            await this.writeTypes(outputDir, types);
        }
        if (errors != null) {
            await this.writeErrors(outputDir, errors);
        }
    }
    // -----------------
    // Instructions
    // -----------------
    async writeInstructions(outputDir, instructions) {
        const instructionsDir = path_1.default.join(outputDir.toString(), 'instructions');
        await (0, utils_1.prepareTargetDir)(instructionsDir);
        (0, utils_1.logInfo)('Writing instructions to directory: %s', instructionsDir);
        for (const [name, code] of Object.entries(instructions)) {
            (0, utils_1.logDebug)('Writing instruction: %s', name);
            await fs_1.promises.writeFile(path_1.default.join(instructionsDir, `${name}.ts`), code, 'utf8');
        }
        (0, utils_1.logDebug)('Writing index.ts exporting all instructions');
        const indexCode = renderIndex(Object.keys(instructions).sort());
        await fs_1.promises.writeFile(path_1.default.join(instructionsDir, `index.ts`), indexCode, 'utf8');
    }
    // -----------------
    // Accounts
    // -----------------
    async writeAccounts(outputDir, accounts) {
        const accountsDir = path_1.default.join(outputDir.toString(), 'accounts');
        await (0, utils_1.prepareTargetDir)(accountsDir);
        (0, utils_1.logInfo)('Writing accounts to directory: %s', accountsDir);
        for (const [name, code] of Object.entries(accounts)) {
            (0, utils_1.logDebug)('Writing account: %s', name);
            await fs_1.promises.writeFile(path_1.default.join(accountsDir, `${name}.ts`), code, 'utf8');
        }
        (0, utils_1.logDebug)('Writing index.ts exporting all accounts');
        const indexCode = renderIndex(Object.keys(accounts).sort());
        await fs_1.promises.writeFile(path_1.default.join(accountsDir, `index.ts`), indexCode, 'utf8');
    }
    // -----------------
    // Types
    // -----------------
    async writeTypes(outputDir, types) {
        const typesDir = path_1.default.join(outputDir.toString(), 'types');
        await (0, utils_1.prepareTargetDir)(typesDir);
        (0, utils_1.logInfo)('Writing types to directory: %s', typesDir);
        for (const [name, code] of Object.entries(types)) {
            (0, utils_1.logDebug)('Writing type: %s', name);
            await fs_1.promises.writeFile(path_1.default.join(typesDir, `${name}.ts`), code, 'utf8');
        }
        (0, utils_1.logDebug)('Writing index.ts exporting all types');
        const indexCode = renderIndex(Object.keys(types).sort());
        await fs_1.promises.writeFile(path_1.default.join(typesDir, `index.ts`), indexCode, 'utf8');
    }
    // -----------------
    // Errors
    // -----------------
    async writeErrors(outputDir, errorsCode) {
        const errorsDir = path_1.default.join(outputDir.toString(), 'errors');
        await (0, utils_1.prepareTargetDir)(errorsDir);
        (0, utils_1.logInfo)('Writing errors to directory: %s', errorsDir);
        (0, utils_1.logDebug)('Writing index.ts containing all errors');
        await fs_1.promises.writeFile(path_1.default.join(errorsDir, `index.ts`), errorsCode, 'utf8');
    }
}
exports.Solita = Solita;
//# sourceMappingURL=solita.js.map