"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.anchorDiscriminatorType = exports.anchorDiscriminatorField = exports.instructionDiscriminator = exports.SIGHASH_GLOBAL_NAMESPACE = exports.accountDiscriminator = exports.ACCOUNT_DISCRIMINATOR_SIZE = exports.UnreachableCaseError = exports.prepareTargetDir = exports.logTrace = exports.logDebug = exports.logInfo = exports.logError = void 0;
const fs_1 = require("fs");
const debug_1 = __importDefault(require("debug"));
const path_1 = __importDefault(require("path"));
const js_sha256_1 = require("js-sha256");
const camelcase_1 = __importDefault(require("camelcase"));
const snake_case_1 = require("snake-case");
exports.logError = (0, debug_1.default)('solita:error');
exports.logInfo = (0, debug_1.default)('solita:info');
exports.logDebug = (0, debug_1.default)('solita:debug');
exports.logTrace = (0, debug_1.default)('solita:trace');
/**
 * Ensures that the given directory exists by creating it recursively when necessary.
 * It also removes all existing files from the directory (non-recursively).
 *
 * @throws Error if the path already exists and is not a directory
 * @category utils
 * @private
 */
async function prepareTargetDir(dir) {
    await ensureDir(dir);
    await cleanDir(dir);
}
exports.prepareTargetDir = prepareTargetDir;
async function ensureDir(dir) {
    if (!(await canAccess(dir))) {
        await fs_1.promises.mkdir(dir, { recursive: true });
        return;
    }
    // dir already exists, make sure it isn't a file
    const stat = await fs_1.promises.stat(dir);
    if (!stat.isDirectory()) {
        throw new Error(`'${dir}' is not a directory`);
    }
}
async function cleanDir(dir) {
    const files = await fs_1.promises.readdir(dir);
    const unlinks = files.map((filename) => fs_1.promises.unlink(path_1.default.join(dir.toString(), filename)));
    return Promise.all(unlinks);
}
async function canAccess(p) {
    try {
        await fs_1.promises.access(p);
        return true;
    }
    catch (_) {
        return false;
    }
}
class UnreachableCaseError extends Error {
    constructor(value) {
        super(`Unreachable case: ${value}`);
    }
}
exports.UnreachableCaseError = UnreachableCaseError;
// -----------------
// Discriminators
// -----------------
/**
 * Number of bytes of the account discriminator.
 */
exports.ACCOUNT_DISCRIMINATOR_SIZE = 8;
/**
 * Calculates and returns a unique 8 byte discriminator prepended to all
 * accounts.
 *
 * @param name The name of the account to calculate the discriminator.
 */
function accountDiscriminator(name) {
    return Buffer.from(js_sha256_1.sha256.digest(`account:${(0, camelcase_1.default)(name, { pascalCase: true })}`)).slice(0, exports.ACCOUNT_DISCRIMINATOR_SIZE);
}
exports.accountDiscriminator = accountDiscriminator;
/**
 * Namespace for global instruction function signatures (i.e. functions
 * that aren't namespaced by the state or any of its trait implementations).
 */
exports.SIGHASH_GLOBAL_NAMESPACE = 'global';
/**
 * Calculates and returns a unique 8 byte discriminator prepended to all instruction data.
 *
 * @param name The name of the instruction to calculate the discriminator.
 */
function instructionDiscriminator(name) {
    return sighash(exports.SIGHASH_GLOBAL_NAMESPACE, name);
}
exports.instructionDiscriminator = instructionDiscriminator;
function sighash(nameSpace, ixName) {
    let name = (0, snake_case_1.snakeCase)(ixName);
    let preimage = `${nameSpace}:${name}`;
    return Buffer.from(js_sha256_1.sha256.digest(preimage)).slice(0, 8);
}
function anchorDiscriminatorField(name) {
    const ty = { array: ['u8', 8] };
    return { name, type: ty };
}
exports.anchorDiscriminatorField = anchorDiscriminatorField;
function anchorDiscriminatorType(typeMapper, context) {
    const ty = { array: ['u8', 8] };
    return typeMapper.map(ty, context);
}
exports.anchorDiscriminatorType = anchorDiscriminatorType;
//# sourceMappingURL=utils.js.map